import re
import os
import sys

utilizacao = "Utilização: python " + os.path.basename(__file__) + " <comando> <caminho da pasta com os manifests>\n\
Comandos:\n all : Imprime todas as permissões + permissões únicas + permissões comuns\n\
 t   : Imprime todas as permissões\n\
 uc  : Imprime permissões únicas + permissões comuns"


# Imprime todas as permissões por APK
def imprime_permissoes_por_apk(objeto):
    print("\n==================\n\nPermissões por APK\n\n==================\n")
    for i in objeto:
        print(re.findall(r'_(.*?)\.xml', i)[0] + ": ",objeto[i])


# Cria objeto com permissões únicas e compartilhadas
def permissoes_unicas_compartilhadas(objeto):
    shared = False
    for i in objeto:
        if not shared:
            shared = objeto[i].copy()
        else:
            shared = list(set(shared) & set(objeto[i]))
    for i in objeto:
        for j in objeto:
            if i != j:
                objeto[i] = list(set(objeto[i]).difference(set(objeto[j])))
    return {'shared': shared.copy(), 'unique': objeto.copy()}


# Imprime lista de permissões comuns das APKs e permissões únicas das mesmas
def imprime_permissoes_unicas_compartilhadas(objeto):
    retorno = permissoes_unicas_compartilhadas(objeto)
    print("\n==========================\n\nPermissões únicas por APKs\n\n==========================\n")
    for el in retorno["unique"]:
        print(re.findall(r'_(.*?)\.xml', el)[0] + ": ",retorno["unique"][el])
    print("\n=========================\n\nPermissões comuns das APK\n\n=========================\n")
    print(retorno["shared"])


# Retorna a lista de permissões por APK baseada na pasta passada por parâmetro
def create_permissions():
    try:
        # Cria uma coleção vazia
        lists = {}
        # Itera todos os arquivos da pasta passada por argumento
        for file in os.listdir(sys.argv[2]):
            # Verifica se o arquivo atual segue o padrão "AndroidManifest_[...].xml"
            if re.match(r'AndroidManifest_(.*?).xml', file):
                # Abre o arquivo
                with open(os.path.join(sys.argv[2], file)) as fd:
                    doc = fd.read()
                # Adiciona as permissões atuais a coleção
                full_permissions = " ".join(re.findall(r'<uses-permission android:name=\"(.*?)\"', doc))
                lists[file] = re.findall(r'\W([A-Z_]*?)\W', full_permissions)
        return lists
    # Exceção de erro de index
    except IndexError:
        print("Insira a pasta com os manifestos!")
        print(utilizacao)
        exit()
    # Pasta não encontrada
    except FileNotFoundError:
        print("Pasta não encontrada!")
        print(utilizacao)
        exit()


def main():
    try:
        if sys.argv[1] == "all":
            lists = create_permissions()
            imprime_permissoes_por_apk(lists.copy())
            imprime_permissoes_unicas_compartilhadas(lists.copy())
        elif sys.argv[1] == "t":
            lists = create_permissions()
            imprime_permissoes_por_apk(lists.copy())
        elif sys.argv[1] == "uc":
            lists = create_permissions()
            imprime_permissoes_unicas_compartilhadas(lists.copy())
        else:
            print(utilizacao)
    # Exceção de erro de index
    except IndexError:
        print("Utilização incorreta!")
        print(utilizacao)
        

if __name__ == "__main__":
    main()
